from typing import List, Set, Optional, Tuple
from dataclasses import dataclass


@dataclass
class Size:
    sizes: List[int]

    def __len__(self):
        return len(self.sizes)

    def __getitem__(self, item) -> int:
        return self.sizes[item]

    def __eq__(self, other) -> bool:
        return isinstance(other, Size) and self.sizes == other.sizes

    def designation(self) -> str:
        if not len(self.sizes):
            return ''

        num_sizes: Set[int] = set(self.sizes)
        if len(num_sizes) <= 1:
            return str(self.sizes[0])

        max_size: int = max(self.sizes)
        max_size_space: int = len(str(max_size))
        return ''.join([str(size).zfill(max_size_space) for size in self.sizes])

    @staticmethod
    def from_designation(designation: str, number_of_channels: int) -> Optional[Tuple['Size', str]]:
        """
        While this can be ambiguous, wer can assume some prevalent sizes.
        """
        if number_of_channels == 0:
            return Size([]), designation

        if not len(designation):
            return None

        if designation[0] == '_':
            designation = designation[1:]

        number_substring: str = ''
        for i in designation:
            if not i.isdigit():
                break
            number_substring += i
        remaining_substring = designation[len(number_substring):]

        len_number_substring: int = len(number_substring)
        if not len_number_substring:
            return None

        # In this case, it is obviously the same size for every channel.
        if len_number_substring % number_of_channels != 0:
            return Size([int(number_substring), ] * number_of_channels), remaining_substring

        # It is unclear if it is the same size for every channel,
        number_of_digits_per_channel = len_number_substring // number_of_channels

        channels: List[int] = []
        for i_channel in range(number_of_channels):
            size = int(number_substring[number_of_digits_per_channel * i_channel:
                                        number_of_digits_per_channel * (i_channel + 1)])
            channels.append(size)

        all_suitable = len(str(max(channels))) == number_of_digits_per_channel

        # It is either: channels or number_of_channels * (int(number_substring), ).
        if number_substring.startswith('0') or number_of_channels == 1 or all_suitable:
            return Size(channels), remaining_substring

        return Size([int(number_substring), ] * number_of_channels), remaining_substring
