from typing import Dict, Optional, List

from dataspree.inference.core.config import ConfigFieldTypes
from dataspree.inference.sensor.data import Camera
from dataspree.inference.sensor.driver.harvesters_driver import HarvestersDriver
from dataspree.inference.sensor.node.camera_input import CameraInput, TemporaryInformationCamera


class HarvestersInput(CameraInput[Camera, HarvestersDriver, TemporaryInformationCamera]):

    def __init__(
            self, name: str, cti_file: str = '/opt/baumer-gapi-sdk-c/lib/libbgapi2_usb.cti',
            required_sensors: Optional[List[str]] = None,
            sensors_auto_acquisition: Optional[List[str]] = None,
            delay: float = 0.1,
            configuration_root_dir: Optional[str] = None,
            auto_connect_all: bool = True
    ) -> None:
        CameraInput.__init__(self, name, driver=HarvestersDriver(cti_file),
                             required_sensors=required_sensors,
                             sensors_auto_acquisition=sensors_auto_acquisition,
                             delay=delay, configuration_root_dir=configuration_root_dir,
                             auto_connect_all=auto_connect_all)

    @classmethod
    def get_config_entries(cls) -> Dict:
        camera_input = CameraInput.get_config_entries()
        camera_input['fields'].append(
            {
                'name': 'cti_file',
                'type': ConfigFieldTypes.STRING_INPUT,
                'display_name': 'CTI file',
                'description': 'Path to CTI file.',
                'default': '/opt/baumer-gapi-sdk-c/lib/libbgapi2_usb.cti'
            }
        )
        camera_input['display_name'] = 'Keyence VJ camera plugin'
        camera_input['name'] = 'keyence_input'
        return camera_input
